################################################################################
########################### Censored T Mixed Models ############################
################################################################################

## Nonlinear case ##

## EM Algorithm / Matrix Q (Hessian Matrix) / Q-function ##

################################################################################
## Moments of certain truncated multivariate-t distributions
################################################################################

cdfNI<-function(x,mu,sigma2,nu,type="Normal"){
resp<-matrix(0,length(x),1)


if(type=="Normal"){
resp<-pnorm(x,mu,sqrt(sigma2))
                  }

if(type=="T"){
z=(x-mu)/sqrt(sigma2)
resp=pt(z,df=nu)
             }
return(resp)
}

TT.moment = function(a,b,R,nu){

  require(mvtnorm)

  GB = GenzBretz(maxpts = 5e4, abseps = 1e-9, releps = 0)

  p = length(a)

  if(p==1){

  if(a== -Inf)  a <- -1e12
  if(b== Inf)   b <- 1e12


G1<- 0.5*(gamma((nu-1)/2)*nu^(nu/2))/((cdfNI(b,0,1,nu,"T")-cdfNI(a,0,1,nu,"T"))*gamma(nu/2)*gamma(1/2))
EX<- ((G1*((nu+a^2)^(-(nu-1)/2)-(nu+b^2)^(-(nu-1)/2))))
EXX<- nu/(nu-2)+(G1*(a*(nu+a^2)^(-(nu-1)/2)-b*(nu+b^2)^(-(nu-1)/2)))

          }

          else{


  a = ifelse(a==-Inf,rep(-1e12,p),a)

  b = ifelse(b== Inf,rep( 1e12,p),b)

  al0 = pmvt(lower = a, upper = b, sigma = R, df = nu, algorithm = GB)[1]

### pdf & cdf

  la1 = (nu-2)/nu; la2 = (nu-4)/nu

  da = (nu-1)/(nu+a^2); db = (nu-1)/(nu+b^2)

  f1a = sqrt(la1)*dt(sqrt(la1)*a,df=nu-2)

  f1b = sqrt(la1)*dt(sqrt(la1)*b,df=nu-2)

  f2 = matrix(NA, p, p)

  G1a = G1b = rep(NA, p)

  G2 = matrix(NA, p, p)



  H = matrix(0,p,p)

  for(r in 1:(p-1))

  {

    temp = R[-r,r]

    S1 = R[-r,-r] - temp %*% t(R[r,-r])

    mua = temp * a[r]; low = a[-r]-mua; upp = b[-r]-mua

    G1a[r] = ifelse(p==2,pt(upp/sqrt(S1/da[r]),df=nu-1)-pt(low/sqrt(S1/da[r]),df=nu-1)

        ,pmvt(lower = low, upper = upp, sigma = S1/da[r], df = nu-1, algorithm = GB)[1])

    mub = temp * b[r]; low = a[-r]-mub; upp = b[-r]-mub

    G1b[r] = ifelse(p==2,pt(upp/sqrt(S1/db[r]),df=nu-1)-pt(low/sqrt(S1/db[r]),df=nu-1)

        ,pmvt(lower = low, upper = upp, sigma = S1/db[r], df = nu-1, algorithm = GB)[1])

    for(s in (r+1):p)

    {

      rs = c(r,s)

      pdf.aa = dmvt(c(a[r],a[s]),sigma=R[rs,rs]/la2,df=nu-4, log =F)

      pdf.ab = dmvt(c(a[r],b[s]),sigma=R[rs,rs]/la2,df=nu-4, log =F)

      pdf.ba = dmvt(c(b[r],a[s]),sigma=R[rs,rs]/la2,df=nu-4, log =F)

      pdf.bb = dmvt(c(b[r],b[s]),sigma=R[rs,rs]/la2,df=nu-4, log =F)

      if(p==2){cdf.aa=cdf.ab=cdf.ba=cdf.bb=1}

      if(p>2)

      {

        tmp = R[-rs,rs]%*%solve(R[rs,rs])

        mu.aa = c(tmp%*%c(a[r],a[s]))

        mu.ab = c(tmp%*%c(a[r],b[s]))

        mu.ba = c(tmp%*%c(b[r],a[s]))

        mu.bb = c(tmp%*%c(b[r],b[s]))

        daa = (nu-2)/(nu+(a[r]^2-2*R[r,s]*a[r]*a[s]+a[s]^2)/(1-R[r,s]^2))

        dab = (nu-2)/(nu+(a[r]^2-2*R[r,s]*a[r]*b[s]+b[s]^2)/(1-R[r,s]^2))

        dba = (nu-2)/(nu+(b[r]^2-2*R[r,s]*b[r]*a[s]+a[s]^2)/(1-R[r,s]^2))

        dbb = (nu-2)/(nu+(b[r]^2-2*R[r,s]*b[r]*b[s]+b[s]^2)/(1-R[r,s]^2))

        R21 = R[-rs,-rs] - R[-rs,rs]%*%solve(R[rs,rs]) %*% R[rs,-rs]

        cdf.aa = ifelse(p==3,pt((b[-rs]-mu.aa)/sqrt(R21/daa),df=nu-2)-pt((a[-rs]-mu.aa)/sqrt(R21/daa),df=nu-2)

            ,pmvt(lower = a[-rs]-mu.aa, upper = b[-rs]-mu.aa, sigma = R21/daa, df=nu-2, algorithm = GB)[1])

        cdf.ab = ifelse(p==3,pt((b[-rs]-mu.ab)/sqrt(R21/dab),df=nu-2)-pt((a[-rs]-mu.ab)/sqrt(R21/dab),df=nu-2)

            ,pmvt(lower = a[-rs]-mu.ab, upper = b[-rs]-mu.ab, sigma = R21/dab, df=nu-2, algorithm = GB)[1])

        cdf.ba = ifelse(p==3,pt((b[-rs]-mu.ba)/sqrt(R21/dba),df=nu-2)-pt((a[-rs]-mu.ba)/sqrt(R21/dba),df=nu-2)

            ,pmvt(lower = a[-rs]-mu.ba, upper = b[-rs]-mu.ba, sigma = R21/dba, df=nu-2, algorithm = GB)[1])

        cdf.bb = ifelse(p==3,pt((b[-rs]-mu.bb)/sqrt(R21/dbb),df=nu-2)-pt((a[-rs]-mu.bb)/sqrt(R21/dbb),df=nu-2)

            ,pmvt(lower = a[-rs]-mu.bb, upper = b[-rs]-mu.bb, sigma = R21/dbb, df=nu-2, algorithm = GB)[1])

      }

      H[r,s] = H[s,r] = pdf.aa*cdf.aa - pdf.ab*cdf.ab - pdf.ba*cdf.ba + pdf.bb*cdf.bb

    }

  }

  ##last part do loop
  r <- p
  temp = R[-r,r]

  S1 = R[-r,-r] - temp %*% t(R[r,-r])

  mua = temp * a[r]; low = a[-r]-mua; upp = b[-r]-mua

  G1a[r] = ifelse(p==2,pt(upp/sqrt(S1/da[r]),df=nu-1)-pt(low/sqrt(S1/da[r]),df=nu-1)

        ,pmvt(lower = low, upper = upp, sigma = S1/da[r], df = nu-1, algorithm = GB)[1])

  mub = temp * b[r]; low = a[-r]-mub; upp = b[-r]-mub

  G1b[r] = ifelse(p==2,pt(upp/sqrt(S1/db[r]),df=nu-1)-pt(low/sqrt(S1/db[r]),df=nu-1)

        ,pmvt(lower = low, upper = upp, sigma = S1/db[r], df = nu-1, algorithm = GB)[1])

  qa = f1a*G1a; qb = f1b*G1b

  EX = c(R %*% (qa-qb)) / al0 / la1

  H = H / la2

  D = matrix(0,p,p)

  diag(D) = a * qa - b * qb - diag(R%*%H)

  al1 = pmvt(lower = a, upper = b, sigma = R/la1, df=nu-2, algorithm = GB)[1]

  EXX = (al1 * R + R %*% (H + D) %*% R) / al0 / la1

  }

return(list(EX=EX,EXX=EXX))

}


##Calculate the first to moments when mu not 0 and Sigma not R

Mtmvt <- function(mu,Sigma,nu,lower,upper){

p=length(lower)

if(p==1){

if(lower== -Inf)  lower <- -1e12
if(upper== Inf)  upper <- 1e12

a1<-(lower-mu)/sqrt(Sigma)
b1<-(upper-mu)/sqrt(Sigma)
M <- TT.moment(a1, b1, 1, nu)
Ey<- mu+sqrt(Sigma)*M$EX
Eyy<- mu^2+Sigma*M$EXX+2*mu*sqrt(Sigma)*M$EX
Vary<- Eyy - Ey^2
}

else{
          Lambda <- diag(1/sqrt(diag(Sigma)))

   if(length(which(upper == Inf)) != 0)  upper[which(upper == Inf)] <- 1e12
   b <- as.vector(diag(1/sqrt(diag(Sigma))) %*% (upper - mu))

   if(length(which(lower == -Inf)) != 0) lower[which(lower == -Inf)] <- -1e12
   a <- as.vector(diag(1/sqrt(diag(Sigma))) %*% (lower - mu))

   R <- Lambda %*% Sigma %*% Lambda

   M <- TT.moment(a, b, R, nu)

   Ey <- mu + solve(Lambda) %*% M$EX

   Eyy <- mu %*% t(mu) + solve(Lambda) %*% M$EX %*% t(mu) + mu %*% t(M$EX) %*% solve(Lambda) + solve(Lambda) %*% M$EXX %*% solve(Lambda)
   Vary<- Eyy- Ey%*%t(Ey)
}

   return(list(Ey=Ey,Eyy=Eyy,Vary=Vary))
}


################################################################################
## EM Agorithm
################################################################################

EMT_NL<-function(cc,y,x,nj,nu){

GB = GenzBretz(maxpts = 5e4, abseps = 1e-9, releps = 0)

  m<-length(nj)
  N<-sum(nj)

  #valores iniciais
  beta1<-c(1.561,0.478,3.525,1.628,-0.002)#matrix(0,p,1)
  sigmae<- 0.20
  D1<-0.01*diag(2)
  iD1<-solve(D1)
  teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)])
  tetacrit <- c(beta1,sigmae)

  q1<-dim(D1)[2]
  p<-length(beta1)
  m1<-m*p
  m2<-m*q1

  tbi=matrix(0,m2,m)   
  criterio<-1
  count<-0

  while(criterio > 0.001){

    count <- count + 1
    print(count)
    soma1<-matrix(0,q1,q1)
    soma2<-0
    soma3<-matrix(0,p,p)
    soma4<-matrix(0,p,1)
    soma5<- matrix(0,p,p)

    tb1<-tbi

    tbi=matrix(0,m2,m)
    tui=rep(0,m)
    tubi=matrix(0,m2,m)
    tubbi=matrix(0,m2,m2)
    tuybi=matrix(0,N,m2)
    tuyyi=matrix(0,N,N)
    tuyi=matrix(0,N,m)
    tyi=matrix(0,N,m)
    xi=matrix(0,N,m1)
    zi=matrix(0,N,m2)

    ver<-matrix(0,m,1)

    for (j in 1:m ){

      cc1=cc[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
      # cc1=c(0,0,0,0,0)

      y1=y[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
      x1=x[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]

      W=funcW(x1,(tb1[(((j-1)*q1)+1) : (j*q1), j]),beta1) #X
      H=funcH(x1,(tb1[(((j-1)*q1)+1) : (j*q1), j]),beta1) #Z
      yt=y1-nlf(x1,beta1,(tb1[(((j-1)*q1)+1) : (j*q1), j]))+W%*%beta1+H%*%(tb1[(((j-1)*q1)+1) : (j*q1), j])

      x1<-W
      z1<-H
      y1<-yt

      gammai=x1%*%beta1
      Psi<-(sigmae*diag(nj[j])+(z1)%*%D1%*%t(z1))
      delta<- solve((sigmae*iD1+t(z1)%*%(z1)))
      dm<-t(y1-gammai)%*%solve(Psi)%*%(y1-gammai)
      cdm<-as.numeric((nu+nj[j])/(nu+dm))

      if(sum(cc1)==0){

        Psi<-(Psi+t(Psi))/2

        tuy<- (matrix(y1,nj[j],1))*cdm
        tuyy<- (y1%*%t(y1))*cdm
        tu<-cdm
        tub<- (delta%*%t(z1)%*%(tuy-gammai*cdm))
        tubb<- delta%*%(diag(q1)*sigmae+(t(z1)%*%(tuyy-tuy%*%t(gammai)-gammai%*%t(tuy)+cdm*gammai%*%t(gammai))%*%z1%*%delta))
        tuyb<- ((tuyy-tuy%*%t(gammai))%*%z1%*%delta)
        tb<- (delta%*%t(z1)%*%(matrix(y1,nj[j],1)-gammai))
        ty<- (matrix(y1,nj[j],1))

        etuy2<- (y1%*%t(y1))*(cdm^2)
        etuy <- (matrix(y1,nj[j],1))*(cdm^2)
        etuy0<- (cdm^2)

        vari2<-etuy2-etuy%*%t(gammai)-gammai%*%t(etuy)+etuy0*(gammai%*%t(gammai))
        vari<-(tuy-(tu*gammai))%*%t(tuy-(tu*gammai))

        ver[j,]<- dmvt(as.vector(y1),as.vector(gammai),as.matrix(Psi),df=nu,log=FALSE)
        #ver[j,]<- dmt(as.vector(y1),as.vector(gammai),Psi,df=nu)

      }

         if(sum(cc1)>0){

         if(sum(cc1)==nj[j]){

          Psi<-(Psi+t(Psi))/2

          muUi<-gammai
          SigmaUi<-Psi
          SigmaUiA<-SigmaUi*nu/(nu+2)
          auxupper<-y1-muUi
          auxU1<-pmvt(upper = c(auxupper), sigma = SigmaUiA, df = nu+2,algorithm = GB)[1]
          auxU2<-pmvt(upper = c(auxupper), sigma = SigmaUi, df = nu,algorithm = GB) [1]
          MoMT = Mtmvt(muUi,SigmaUiA,nu+2,rep(-Inf,nj[j]),y1)
          MoMT1 = Mtmvt(muUi,SigmaUi,nu,rep(-Inf,nj[j]),y1)
          U0<-as.numeric(auxU1/auxU2)
          U1<-auxU1/auxU2*MoMT$Ey
          U2<-auxU1/auxU2*MoMT$Eyy

          tuy<-U1
          tuyy<-U2
          tu<-U0
          tub<-delta%*%t(z1)%*%(U1-gammai*U0)
          tubb<-delta%*%(diag(q1)*sigmae+t(z1)%*%(U2-U1%*%t(gammai)-gammai%*%t(U1)+U0*gammai%*%t(gammai))%*%z1%*%delta)
          tuyb<-(U2-U1%*%t(gammai))%*%z1%*%delta
          tb<- (delta%*%t(z1)%*%(matrix(MoMT1$Ey,nj[j],1)-gammai))
          ty<-MoMT1$Ey

          cp<-(((nu+nj[j])/nu)^2)*((gamma((nu+nj[j])/2)*gamma((nu+4)/2))/(gamma(nu/2)*gamma((nu+nj[j]+4)/2)))

          SigmaUiAey<-SigmaUi*nu/(nu+4)
          auxU1ey<-pmvt(upper = c(auxupper), sigma = SigmaUiAey, df = nu+4,algorithm = GB)[1]
          MoMTey = Mtmvt(muUi,SigmaUiAey,nu+4,rep(-Inf,nj[j]),y1)

          etuy2<- cp*(auxU1ey/auxU2)*MoMTey$Eyy
          etuy<- cp*(auxU1ey/auxU2)*MoMTey$Ey
          etuy0<- cp*(auxU1ey/auxU2)

          vari2<-etuy2-etuy%*%t(gammai)-gammai%*%t(etuy)+etuy0*(gammai%*%t(gammai))
          vari<-(tuy-(tu*gammai))%*%t(tuy-(tu*gammai))

          ver[j,]<- pmvt(upper = c(auxupper), sigma = SigmaUi, df = nu,algorithm = GB) [1]


        }

        else {

          PsiA<-Psi*nu/(nu+2)
          nu1<-(nu+length(cc1[cc1==0]))

          muc<-x1[cc1==1,]%*%beta1+Psi[cc1==1,cc1==0]%*%solve(Psi[cc1==0,cc1==0])%*%(y1[cc1==0]-x1[cc1==0,]%*%beta1)
          Sc <-Psi[cc1==1,cc1==1]-Psi[cc1==1,cc1==0]%*%solve(Psi[cc1==0,cc1==0])%*%Psi[cc1==0,cc1==1]
          ScA<-nu/(nu+2)*Sc

          Qy1<-t(y1[cc1==0]-x1[cc1==0,]%*%beta1)%*%solve(Psi[cc1==0,cc1==0])%*%(y1[cc1==0]-x1[cc1==0,]%*%beta1)
          Qy2<-t(y1[cc1==0]-x1[cc1==0,]%*%beta1)%*%solve(PsiA[cc1==0,cc1==0])%*%(y1[cc1==0]-x1[cc1==0,]%*%beta1)

          auxcte<-as.numeric((nu+Qy1)/(nu+length(cc1[cc1==0])))
          auxcte1<-as.numeric((nu+2+Qy2)/(nu+2+length(cc1[cc1==0])))

          Sc22<-auxcte*Sc

          muUi<-muc
          SigmaUi<-Sc22

          SigmaUiA<- auxcte1*ScA
          auxupper<- y1[cc1==1]-muUi


          auxU1<-pmvt(upper = c(auxupper), sigma = SigmaUiA, df = nu1+2, algorithm = GB)[1]
          auxU2<-pmvt(upper = c(auxupper), sigma = SigmaUi, df = nu1, algorithm = GB)[1]

          MoMT = Mtmvt(muUi,SigmaUiA,nu1+2,rep(-Inf,length(cc1[cc1==1])),y1[cc1==1])

          MoMT1 = Mtmvt(muUi,SigmaUi,nu1,rep(-Inf,length(cc1[cc1==1])),y1[cc1==1])

          U0<-as.numeric(auxU1/auxU2)/auxcte
          U1<-(U0)*(MoMT$Ey)
          U2<-(U0)*(MoMT$Eyy)

          Auxtuy<-(matrix(y1,nj[j],1))

          tuy<- Auxtuy*U0
          tuy[cc1==1]<- U1

          tuyy<-(Auxtuy%*%t(Auxtuy))

          AAx<-tuyy[cc1==0,cc1==0]*U0
          ABx<-Auxtuy[cc1==0]%*%t(U1)
          BAx<-t(ABx)
          BBx<-U2

          tuyy[cc1==0,cc1==0]<- AAx
          tuyy[cc1==0,cc1==1]<- ABx
          tuyy[cc1==1,cc1==0]<- BAx
          tuyy[cc1==1,cc1==1]<- BBx

          tu<-U0
          tub<-delta%*%t(z1)%*%(tuy-gammai*tu)
          tubb<-delta%*%(diag(q1)*sigmae+t(z1)%*%(tuyy-tuy%*%t(gammai)-gammai%*%t(tuy)+tu*gammai%*%t(gammai))%*%z1%*%delta)
          tuyb<-(tuyy-tuy%*%t(gammai))%*%z1%*%delta

          eee<-matrix(y1,nj[j],1)
          eee[cc1==1]<-MoMT1$Ey
          tb<- (delta%*%t(z1)%*%(eee-gammai))

          ty<- Auxtuy
          ty[cc1==1]<-MoMT1$Ey

          SigmaUiAey<- as.numeric((nu+Qy1)/(nu+4+length(cc1[cc1==0])))*Sc

          auxU1ey<-pmvt(upper = c(auxupper), sigma = SigmaUiAey, df = nu1+4, algorithm = GB)[1]

          MoMTey = Mtmvt(muUi,SigmaUiAey,nu1+4,rep(-Inf,length(cc1[cc1==1])),y1[cc1==1])

          dp<-((nu+nj[j])^2)*((gamma((nu+nj[j])/2)*gamma((nu+4+length(cc1[cc1==0]))/2))/
                  (gamma((nu+length(cc1[cc1==0]))/2)*gamma((nu+4+nj[j])/2)))

          U0ey<-as.numeric((auxU1ey/auxU2)*(dp/((nu+Qy1)^2)))
          U1ey<-(U0ey)*(MoMTey$Ey)
          U2ey<-(U0ey)*(MoMTey$Eyy)

          Auxtuyey<-(matrix(y1,nj[j],1))

          etuy2<-(Auxtuyey%*%t(Auxtuyey))

          AAxey<-etuy2[cc1==0,cc1==0]*U0ey
          ABxey<-Auxtuyey[cc1==0]%*%t(U1ey)
          BAxey<-t(ABxey)
          BBxey<-U2ey

          etuy2[cc1==0,cc1==0]<- AAxey
          etuy2[cc1==0,cc1==1]<- ABxey
          etuy2[cc1==1,cc1==0]<- BAxey
          etuy2[cc1==1,cc1==1]<- BBxey

          etuy<- Auxtuyey*U0ey
          etuy[cc1==1]<-U1ey

          etuy0<-U0ey

          vari2<-etuy2-etuy%*%t(gammai)-gammai%*%t(etuy)+etuy0*(gammai%*%t(gammai))
          vari<-(tuy-(tu*gammai))%*%t(tuy-(tu*gammai))

          SigmaUi<-(SigmaUi+t(SigmaUi))/2
          Psi[cc1==0,cc1==0]<-(Psi[cc1==0,cc1==0]+t(Psi[cc1==0,cc1==0]))/2

          ver[j,]<-dmvt(y1[cc1==0],gammai[cc1==0],as.matrix(Psi[cc1==0,cc1==0]),df=nu,log=FALSE)*(pmvt(upper = c(auxupper), sigma = SigmaUi, df = nu1, algorithm = GB)[1])
   
        }

      }

      soma1<- soma1 + tubb
      soma2<- soma2 + (sum(diag(tuyy))-t(tuy)%*%gammai-t(gammai)%*%tuy-sum(diag(t(tuyb)%*%z1))-sum(diag(tuyb%*%t(z1)))
                          +t(gammai)%*%z1%*%tub+t(tub)%*%t(z1)%*%gammai+tu*(t(gammai)%*%gammai)+sum(diag(tubb%*%t(z1)%*%z1)))
      soma3<- soma3 + (tu*(t(x1)%*%x1))
      soma4<- soma4 + (t(x1)%*%(tuy-z1%*%tub))
      soma5<- soma5 + ((((nu+nj[j])/(nu+nj[j]+2))*t(x1)%*%solve(Psi)%*%x1)-(t(x1)%*%solve(Psi)%*%(vari2-vari)%*%solve(Psi)%*%x1))

      tbi[(((j-1)*q1)+1) : (j*q1), j]<-tb
      tuyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (sum(nj[1:j-1])+1) : (sum(nj[1:j]))]<-tuyy
      tuyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), j]<-tuy
      tuybi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)]<-tuyb
      tubi[(((j-1)*q1)+1) : (j*q1), j]<-tub
      tubbi[(((j-1)*q1)+1) : (j*q1), (((j-1)*q1)+1) : (j*q1)]<-tubb
      tui[j]<-tu
      zi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)]<-z1
      xi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*p)+1) : (j*p)]<-x1
      tyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), j]<-ty

    }

    beta1<- solve(soma3)%*%soma4
    sigmae<- (1/N)*(soma2)
    sigmae<-as.numeric(sigmae)
    D1<- (1/m)*(soma1)
    iD1<- solve(D1)
    teta1 <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)])
    teta1crit <- c(beta1,sigmae)
    logver <- sum(log(ver))
    varbeta<-solve(soma5)
    
    if (count>1){
      criterio <- sqrt((teta1crit/tetacrit-1)%*%(teta1crit/tetacrit-1))
    }

    if (count==300){
      criterio <- 0.00000000001
    }

    teta<-teta1
    tetacrit<-teta1crit
    logver1<-logver
  }

  dd<-D1[upper.tri(D1, diag = T)]

  npar<-length(c(teta1))

  ni<-sum(nj)

  loglik<-logver1

  AICc<- -2*loglik +2*npar
  AICcorr<- AICc + ((2*npar*(npar+1))/(ni-npar-1))
  BICc <- -2*loglik +log(ni)*npar

  obj.out <- list(beta1 = beta1, sigmae= sigmae, dd = dd, iter = count,  loglik=loglik, AIC=AICc, BIC=BICc, AICcorr=AICcorr, varbeta=varbeta,
                  tbi= tbi, xi = xi, zi = zi, tyi=tyi, tui=tui, tubi = tubi, tubbi = tubbi, tuybi = tuybi, tuyi = tuyi, tuyyi = tuyyi, DD=D1 )

  class(obj.out) <- "EM_NCens"

  return(obj.out)

}

## Q Matrix ##

HessianaQ<-function(cc,y,xi,zi,nj,beta1,sigmae,D1,iD1,tui,tubi,tubbi,tuybi,tuyi,tuyyi){

  m<-length(nj)[1]
  N<-sum(nj)
  q1<-dim(D1)[2]
  p<-length(beta1)
  m1<-m*p
  m2<-m*q1
  
  soma1<-0
  soma2=matrix(0,p,p)
  soma3=matrix(0,p,1)
  
  somas<-matrix(0,3,3)
  
  for (j in 1:m ){
    
    cc1=cc[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
    
    y1=y[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
    z1<-matrix(zi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)],nj[j],q1)
    x1<-matrix(xi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*p)+1) : (j*p)],nj[j],p)
    
    gammai=x1%*%beta1
    
    tub1<- matrix(tubi[(((j-1)*q1)+1) : (j*q1), j],q1,1)
    tubb1<-matrix(tubbi[(((j-1)*q1)+1) : (j*q1), (((j-1)*q1)+1) : (j*q1)],q1,q1)
    tuyb1<-matrix(tuybi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(((j-1)*q1)+1) : (j*q1)],nj[j],q1)
    tuyy1<-matrix(tuyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(sum(nj[1:j-1])+1) : (sum(nj[1:j]))],nj[j],nj[j])
    tuy1<-matrix(tuyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),j],nj[j],1)
    tu1<-as.numeric(tui[j])
    
    soma1<- soma1 + (sum(diag(tuyy1))-t(tuy1)%*%gammai-t(gammai)%*%tuy1-sum(diag(t(tuyb1)%*%z1))-sum(diag(tuyb1%*%t(z1)))
                     +t(gammai)%*%z1%*%tub1+t(tub1)%*%t(z1)%*%gammai+tu1*(t(gammai)%*%gammai)+sum(diag(tubb1%*%t(z1)%*%z1)))
    soma2<- soma2 + (tu1*(t(x1)%*%x1))
    soma3<- soma3 + (t(x1)%*%(tuy1-z1%*%tub1-tu1*x1%*%beta1))
    
    d1<-matrix(rep(0,4),ncol=2,nrow=2);d1[1,1]<-1
    d2<-matrix(rep(0,4),ncol=2,nrow=2);d2[1,2]<-1;d2[2,1]<-1
    d3<-matrix(rep(0,4),ncol=2,nrow=2);d3[2,2]<-1
    
    d<-list(d1,d2,d3)
    
    for(i in 1:3){
      for(j in 1:3){
        di<-d[[i]]
        dj<-d[[j]]
        somas[i,j]<-somas[i,j]+sum(diag(iD1%*%di%*%iD1%*%dj-((iD1%*%di%*%iD1%*%dj%*%iD1+iD1%*%dj%*%iD1%*%di%*%iD1)%*%tubb1)))   
      }} 
    
  }
  
  derbetabeta<-((-1/sigmae)*soma2)
  derbetasigmae<-((-1/(sigmae^2))*soma3)
  derbetaalphas<-matrix(0,nrow=p,ncol=3)
  dersigmaesigmae<-((1/(2*(sigmae^2)))*sum(nj))-((1/(sigmae^3))*soma1)
  dersigmaealphas<-matrix(0,nrow=1,ncol=3)
  deralphaialphaj<-(0.5*somas)
  
  derbeta<-matrix(c(derbetabeta,derbetasigmae,derbetaalphas),nrow=p,ncol=(p+1+3))
  dersigmae<-matrix(c(t(derbetasigmae),dersigmaesigmae,dersigmaealphas),nrow=1,ncol=(p+1+3))
  deralphas<-matrix(c(t(derbetaalphas),t(dersigmaealphas),deralphaialphaj),nrow=3,ncol=(p+1+3))
  
  MatrizQ<-matrix(c(t(derbeta),t(dersigmae),t(deralphas)),nrow=(p+1+3),ncol=(p+1+3))
  MatrizQbeta<-derbetabeta
  MatrizQsigmae<-dersigmaesigmae
  MatrizQalpha<-deralphaialphaj
  
  obj.out <- list(MatrizQ = MatrizQ, MatrizQbeta = MatrizQbeta, MatrizQsigmae = MatrizQsigmae, MatrizQalpha = MatrizQalpha)
  
  return(obj.out)

}

## Q-function ##

Q.function<-function(cc,y,x,z,nj,beta1,sigmae,D1,iD1,tui,tubi,tubbi,tuybi,tuyi,tuyyi){

  m<-length(nj)[1]
  N<-sum(nj)
  p<-dim(x)[2]
  q1<-dim(z)[2]
  m1<-m*p
  m2<-m*q1

  soma1<-soma2<-0

  for (j in 1:m ){

    cc1=cc[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]

    y1=y[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
    z1<-matrix(zi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)],nj[j],q1)
    x1<-matrix(xi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*p)+1) : (j*p)],nj[j],p)

    gammai=x1%*%beta1

    tub1<- tubi[(((j-1)*q1)+1) : (j*q1), j]
    tubb1<-tubbi[(((j-1)*q1)+1) : (j*q1), (((j-1)*q1)+1) : (j*q1)]
    tuyb1<-tuybi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(((j-1)*q1)+1) : (j*q1)]
    tuyy1<-tuyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
    tuy1<-tuyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),j]
    tu1<-tui[j]


    soma1<-soma1+sum(diag(tubb1%*%iD1))
    soma2<-soma2+(sum(diag(tuyy1))-t(tuy1)%*%gammai-t(gammai)%*%tuy1-sum(diag(t(tuyb1)%*%z1))-sum(diag(yuyb1%*%t(z1)))
                  +t(gammai)%*%z1%*%tub1+t(tub1)%*%t(z1)%*%gammai+tu1*(t(gammai)%*%gammai)+sum(diag(tubb1%*%t(z1)%*%z1)))

  }

  Q.theta<- -(0.5*sum(nj)*log(sigmae))-(0.5*m*log(det(D1)))-(0.5*(soma1))-((0.5/sigmae)*soma2)

  return(Q.theta)

}

################################################################################
######################## Censored Normal Mixed Models ##########################
################################################################################

## Nonlinear case ##

################################################################################
## Moments of certain Normal multivariate-t distributions
################################################################################

MomemNT<-function(u=c(0,0),S=diag(2),qc=c(1,2)) {
  
  nic=length(u)
  
  if (nic==1) {
    
    qq <- (1/sqrt(S))*(-qc+u)
    R<-1
    alpha <- pnorm(-qq)
    
    dd <- dnorm(-qq)
    H <- qq*dd
    EX <- (1/alpha)*dd   # a vector with a length of nic
    EXX <- 1+1/alpha*H
    varX <- EXX-EX^2
    Eycens <- -sqrt(S)*EX+u
    varyic<- varX*S
    E2yy<-varyic+Eycens^2
    
  }
  
  else {
    
    qq <- diag(1/sqrt(diag(S)))%*%(-qc+u)
    R <-  diag(1/sqrt(diag(S)))%*%S%*%diag(1/sqrt(diag(S)))
    alpha <- pmvnorm(upper=as.vector(-qq), corr=R)
    dd <- rep(0, nic)   #derivative vector
    
    for (j in 1:nic){
      V <- R[-j, -j, drop=F]-R[-j,j, drop=F]%*%R[j,-j, drop=F]
      nu <- -qq[-j]+R[-j,j, drop=F]%*%qq[j]
      dd[j] <- dnorm(-qq[j])*pmvnorm(upper=as.vector(nu), sigma=V)
    }
    
    H <- matrix(rep(0, nic*nic), nrow=nic)
    RH <- matrix(rep(0, nic*nic), nrow=nic)
    
    if(nic==2)     {
      H[1,2] <- H[2,1] <- dmvnorm(-qq[c(1, 2)],sigma=matrix(c(1, R[1,2], R[2,1], 1), nrow=2))
      #sigma==R since qq is standardized
      RH[1,2] <- RH[2,1] <- R[1,2]*H[1,2]
    }
    
    else {
      for( s in 1:(nic-1)){
        for (t in (s+1):nic){
          invR <- solve(R[c(s,t), c(s,t), drop=F])
          nu <- -qq[-c(s,t)]+R[-c(s,t), c(s,t), drop=F]%*%invR%*%qq[c(s,t),,drop=F]
          V <-  R[-c(s,t), -c(s,t), drop=F]- R[-c(s,t), c(s,t), drop=F]%*%invR%*%R[c(s,t), -c(s,t), drop=F]
          H[s,t] <- H[t,s] <- pmvnorm(upper=as.vector(nu), sigma=V)*dmvnorm(-qq[c(s, t)],sigma=matrix(c(1, R[s,t], R[t,s], 1), nrow=2))
          RH[s,t] <- RH[t,s] <- R[s,t]*H[s,t]
        }
      }
    }
    
    h <- qq*dd-apply(RH, 1, sum)
    diag(H) <- h
    EX <- (1/alpha)*R%*%dd   # a vector with a length of nic
    EXX <- R+1/alpha*R%*%H%*%R
    varX <- EXX-EX%*%t(EX)
    Eycens <- -diag(sqrt(diag(S)))%*%EX+u
    varyic <- diag(sqrt(diag(S)))%*%varX%*%diag(sqrt(diag(S)))
    E2yy <- varyic+Eycens%*%t(Eycens)
    
  }
  
  return(list(Ey=Eycens,Eyy=E2yy,Vary=varyic))
  
}

################################################################################
## EM Agorithm 
################################################################################

EMN_NL<-function(cc,y,x,nj){
  
  m<-length(nj)
  N<-sum(nj)
  
  beta1<-c(1.561,0.478,3.525,1.628,-0.002)#matrix(0,p,1)
  sigmae<- 0.20
  D1<-0.01*diag(2)
  iD1<-solve(D1)
  teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)])
  tetacrit <- c(beta1,sigmae)
  
  q1<-dim(D1)[2]
  p<-length(beta1)  
  m1<-m*p
  m2<-m*q1
  
  ubi=matrix(0,m2,m)     #inicial.b
  ubbi=matrix(0,m2,m2)
  uybi=matrix(0,N,m2)
  uyyi=matrix(0,N,N)
  uyi=matrix(0,N,m)
  xi=matrix(0,N,m1)
  zi=matrix(0,N,m2)
  
  criterio<-1
  count<-0
  
  while(criterio > 0.001){
    
    count <- count + 1
    print(count)
    soma1<-matrix(0,q1,q1)
    soma2<-0
    soma3<-matrix(0,p,p)
    soma4<-matrix(0,p,1)
    soma5<-matrix(0,p,p) 
    
    ver<-matrix(0,m,1)
    
    ub1<-ubi
    
    for (j in 1:m ){
      
      cc1=cc[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
      
      y1=y[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
      x1=x[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]

      W=funcW(x1,(ub1[(((j-1)*q1)+1) : (j*q1), j]),beta1) #X
      H=funcH(x1,(ub1[(((j-1)*q1)+1) : (j*q1), j]),beta1) #Z
      yt=y1-nlf(x1,beta1,(ub1[(((j-1)*q1)+1) : (j*q1), j]))+W%*%beta1+H%*%(ub1[(((j-1)*q1)+1) : (j*q1), j])
      
      x1<-W
      z1<-H
      y1<-yt
      
      gammai=x1%*%beta1
      
      if(sum(cc1)==0){
        
        Psi<-(sigmae*diag(nj[j])+(z1)%*%D1%*%t(z1))
        
        delta<- solve(iD1+(t(z1)%*%((z1*(1/sigmae)))))
        uy<- matrix(y1,nj[j],1)
        uyy<- y1%*%t(y1)
        ub<- delta%*%(t(z1)*(1/sigmae))%*%(uy-gammai)
        ubb<- delta+(delta%*%(t(z1)*((1/sigmae)^2))%*%(uyy-uy%*%t(gammai)-gammai%*%t(uy)+gammai%*%t(gammai))%*%z1%*%delta)
        uyb<- (uyy-uy%*%t(gammai))%*%(z1*(1/sigmae))%*%delta
        ver[j,]<- dmvnorm(as.vector(y1),gammai,Psi)
        
      }
      
      if(sum(cc1)>=1){
        
        Psi<-(sigmae*diag(nj[j])+(z1)%*%D1%*%t(z1))
        
        if(sum(cc1)==nj[j]){
          muc=x1%*%beta1
          Sc<-Psi
          delta<- solve(iD1+(t(z1)%*%((z1*(1/sigmae)))))
          aux<- MomemNT(muc,Sc,y1)
          uy<-aux$Ey
          uyy<- aux$Eyy
          ub<- delta%*%(t(z1)*(1/sigmae))%*%(uy-gammai)
          ubb<- delta+(delta%*%(t(z1)*((1/sigmae)^2))%*%(uyy-uy%*%t(gammai)-gammai%*%t(uy)+gammai%*%t(gammai))%*%z1%*%delta)
          uyb<- (uyy-uy%*%t(gammai))%*%(z1*(1/sigmae))%*%delta
          ver[j,]<-pmnorm(y1,muc,Sc)
        }
        
        else {
          muc=x1[cc1==1,]%*%beta1+Psi[cc1==1,cc1==0]%*%solve(Psi[cc1==0,cc1==0])%*%(y1[cc1==0]-x1[cc1==0,]%*%beta1)
          Sc <-Psi[cc1==1,cc1==1]-Psi[cc1==1,cc1==0]%*%solve(Psi[cc1==0,cc1==0])%*%Psi[cc1==0,cc1==1]
          delta<- solve(iD1+(t(z1)%*%((z1*(1/sigmae)))))
          aux <-MomemNT(muc,Sc,y1[cc1==1])
          uy <-matrix(y1,nj[j],1)
          uy[cc1==1]<-aux$Ey
          
          uyy<-matrix(0,nj[j],nj[j])
          uyy[cc1==1,cc1==1]<-aux$Vary
          uyy<- uyy+uy%*%t(uy)
          ub<- delta%*%(t(z1)*(1/sigmae))%*%(uy-gammai)
          ubb<- delta+(delta%*%(t(z1)*((1/sigmae)^2))%*%(uyy-uy%*%t(gammai)-gammai%*%t(uy)+gammai%*%t(gammai))%*%z1%*%delta)
          uyb<- (uyy-uy%*%t(gammai))%*%(z1*(1/sigmae))%*%delta
          ver[j,]<-dmvnorm(y1[cc1==0],gammai[cc1==0],as.matrix(Psi[cc1==0,cc1==0]))*pmnorm(y1[cc1==1],muc,Sc)
        }
        
      }
      
      soma1<- soma1 + ubb
      soma2<- soma2 + (sum(diag(uyy))-t(uy)%*%gammai-t(gammai)%*%uy-sum(diag(t(uyb)%*%z1))-sum(diag(uyb%*%t(z1)))
                       +t(gammai)%*%z1%*%ub+t(ub)%*%t(z1)%*%gammai+t(gammai)%*%gammai+sum(diag(ubb%*%t(z1)%*%z1)))
      soma3<- soma3 + (t(x1)%*%x1)
      soma4<- soma4 + (t(x1)%*%(uy-z1%*%ub))
      soma5<- soma5 + (t(x1)%*%solve(Psi)%*%x1-t(x1)%*%solve(Psi)%*%(uyy-uy%*%t(uy))%*%solve(Psi)%*%x1)
      
      
      uyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (sum(nj[1:j-1])+1) : (sum(nj[1:j]))]<-uyy
      uyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), j]<-uy
      uybi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)]<-uyb
      ubi[(((j-1)*q1)+1) : (j*q1), j]<-ub
      ubbi[(((j-1)*q1)+1) : (j*q1), (((j-1)*q1)+1) : (j*q1)]<-ubb
      zi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*q1)+1) : (j*q1)]<-z1
      xi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])), (((j-1)*p)+1) : (j*p)]<-x1
    }
    
    beta1<- solve(soma3)%*%soma4
    sigmae<- (1/(N))*as.numeric(soma2)
    D1<- (1/(m))*(soma1)
    iD1<-solve(D1)
    teta1 <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)])
    teta1crit <- c(beta1,sigmae)
    varbeta<-solve(soma5)
    logver <- sum(log(ver))
    
    if (count>1){
      criterio <- sqrt((teta1crit/tetacrit-1)%*%(teta1crit/tetacrit-1))
    }

    if (count==300){
      criterio <- 0.00000000001
    }

    teta<-teta1
    tetacrit<-teta1crit
    logver1<-logver
      
  }
  
  dd<-D1[upper.tri(D1, diag = T)]
  
  npar<-length(c(teta1))
  
  ni<-sum(nj)
  
  loglik<-logver1
  
  AICc<- -2*loglik +2*npar
  AICcorr<- AICc + ((2*npar*(npar+1))/(ni-npar-1))
  BICc <- -2*loglik +log(ni)*npar
  
  obj.out <- list(beta1 = beta1, sigmae= sigmae, dd = dd, iter = count, varbeta=varbeta, loglik=loglik, AIC=AICc, BIC=BICc, AICcorr=AICcorr, 
                  xi = xi, zi = zi, ubi = ubi, ubbi = ubbi, uybi = uybi, uyi = uyi, uyyi = uyyi, DD=D1  )
  
  class(obj.out) <- "EM_NCens"
  
  return(obj.out)
  
}


